summaryrefslogtreecommitdiff
path: root/t/ax/test-init.sh
diff options
context:
space:
mode:
authorStefano Lattarini <stefano.lattarini@gmail.com>2012-06-24 10:36:15 +0200
committerStefano Lattarini <stefano.lattarini@gmail.com>2012-06-25 10:29:43 +0200
commitc2a52bf0e8e87b8f347ca22b9ab2da7613bc4336 (patch)
tree1e2115c0d24da906daa64cc84d853e538129574f /t/ax/test-init.sh
parentaa630a928a586fd8dc8a087337c3462d7f4fd1c6 (diff)
downloadautomake-c2a52bf0e8e87b8f347ca22b9ab2da7613bc4336.tar.gz
tests: simpler workaround for shells losing the exit status in exit trap
Now that we can assume our tests are run by a decent POSIX shell, we can simplify our workaround aimed at having the exit status propagated correctly to the code in the exit trap. Unfortunately, we cannot dispense with such a workaround altogether, because it's still required by some shells we need to support (at least Solaris 10 /bin/ksh and /usr/xpg4/bin/sh). For more information about the need of that workaround, see the entry about 'trap' in the section "Limitations of Shell Builtins" in the Autoconf manual: <http://www.gnu.org/software/autoconf/manual/autoconf.html#trap> The new workaround has been tested successfully with the following shells: - Bash 4.1 - Bash 3.2 - Bash 3.0 - Bash 2.05b - dash 0.5.5.1 - dash 0.5.2 - AT&T Ksh 93u (from official Debian package) - MirBSD Korn Shell 40.2 (from official Debian package) - Solaris 9, 10 and 11 /bin/ksh - Solaris 9, 10 and 11 /usr/xpg4/bin/sh - NetBSD 5.1 /bin/sh - NetBSD 5.1 /bin/ksh * t/ax/test-init.sh (Exit): Rename ... (_am_exit): ... like this. (exit): New alias for '_am_exit'. We cannot simply redefine 'exit' as a shell function, because some shells (dash 0.5.5.1, Solaris 10 /bin/ksh and /usr/xpg4/bin/sh) do not allow it. (_am_exit, trap): Add extra escaping for 'exit' calls, to ensure we really invoke the 'exit' builtin and not our alias with the same name. * configure.ac: Check that the shell selected to run our testsuite supports aliases named like shell builtins. * t/REAMDE: Adjust. * All tests: Adjust, by simply using 'exit' instead of 'Exit'. * t/self-check-explicit-skips.sh: Adjust: the first usage of 'exit' after it has been redefined as an alias must be on a new line w.r.t. that where the alias is defined, in order for the redefinition to be honored. * syntax-checks.mk (sc_tests_Exit_not_exit): Delete. (sc_tests_exit_not_Exit): New. (syntax_check_rules): Adjust. (sc_tests_automake_fails): Simplify the recipe a little. * Several tests: Remove now useless spurious quoting once required to placate the 'sc_tests_Exit_not_exit' maintainer check. * gen-testsuite-part: Likewise. Also, avoid uses of 'Exit' in the generated scripts. Signed-off-by: Stefano Lattarini <stefano.lattarini@gmail.com>
Diffstat (limited to 't/ax/test-init.sh')
-rw-r--r--t/ax/test-init.sh29
1 files changed, 16 insertions, 13 deletions
diff --git a/t/ax/test-init.sh b/t/ax/test-init.sh
index 116d4b1b2..24945b67d 100644
--- a/t/ax/test-init.sh
+++ b/t/ax/test-init.sh
@@ -155,22 +155,24 @@ am_keeping_testdirs ()
esac
}
# This is used in 'Exit' and in the exit trap. See comments in the latter
# for more information,
am__test_skipped=no
# We use a trap below for cleanup. This requires us to go through
# hoops to get the right exit status transported through the signal.
-# So use "Exit STATUS" instead of "exit STATUS" inside of the tests.
# Turn off errexit here so that we don't trip the bug with OSF1/Tru64
-# sh inside this function.
-Exit ()
+# sh inside this function (FIXME: is this still relevant now that we
+# require a POSIX shell?).
+_am_exit ()
{
set +e
# See comments in the exit trap for the reason we do this.
test 77 = $1 && am__test_skipped=yes
- (exit $1); exit $1
+ # Spurious escaping to ensure we do not call our 'exit' alias.
+ (\exit $1); \exit $1
}
+alias exit=_am_exit
if test $am_using_tap = yes; then
am_funcs_file=tap-functions.sh
@@ -181,11 +183,11 @@ fi
if test -f "$am_testauxdir/$am_funcs_file"; then
. "$am_testauxdir/$am_funcs_file" || {
echo "$me: error sourcing $am_testauxdir/$am_funcs_file" >&2
- Exit 99
+ exit 99
}
else
echo "$me: $am_testauxdir/$am_funcs_file not found" >&2
- Exit 99
+ exit 99
fi
unset am_funcs_file
@@ -257,7 +259,7 @@ AUTOMAKE_run ()
cat stderr >&2
cat stdout
if test $am_using_tap != yes; then
- test $am__got_rc -eq $am__exp_rc || Exit 1
+ test $am__got_rc -eq $am__exp_rc || exit 1
return
fi
if test -z "$am__desc"; then
@@ -321,12 +323,12 @@ extract_configure_help ()
grep_configure_help ()
{
./configure --help > am--all-help \
- || { cat am--all-help; Exit 1; }
+ || { cat am--all-help; exit 1; }
cat am--all-help
extract_configure_help "$1" am--all-help > am--our-help \
- || { cat am--our-help; Exit 1; }
+ || { cat am--our-help; exit 1; }
cat am--our-help
- $EGREP "$2" am--our-help || Exit 1
+ $EGREP "$2" am--our-help || exit 1
}
# using_gmake
@@ -979,7 +981,8 @@ trap 'exit_status=$?
am_keeping_testdirs || rm_rf_ $testSubDir
set +x
echo "$me: exit $exit_status"
- exit $exit_status
+ # Spurious escaping to ensure we do not call our "exit" alias.
+ \exit $exit_status
' 0
trap "fatal_ 'caught signal SIGHUP'" 1
trap "fatal_ 'caught signal SIGINT'" 2
@@ -155,22 +155,24 @@ am_keeping_testdirs ()
esac
}
-# This is used in 'Exit' and in the exit trap. See comments in the latter
-# for more information,
+# This is used in '_am_exit' and in the exit trap. See comments in
+# the latter for more information.
am__test_skipped=no
# We use a trap below for cleanup. This requires us to go through